; Lorna Ellis
; 07/11/2019
; read_beacon_browse.pro

; This is utility to read PLASTIC Beacon Browse CDFs.

PRO read_beacon_browse

version = 1
tab = string(9B)

; read settings file
settings_file = 'read_beacon_browse.settings'
openr, settings_lun, settings_file, /get_lun
line = ' '
FOR i = 0, 4 DO BEGIN 
    readf, settings_lun, line
ENDFOR 
readf, settings_lun, directory_mode
IF (directory_mode NE 2) AND (directory_mode NE 1) THEN BEGIN
    print, 'ERROR: invalid directory mode'
    stop
ENDIF 
IF directory_mode EQ 1 THEN BEGIN ; directory mode
    FOR i = 0, 2 DO BEGIN 
        readf, settings_lun, line
    ENDFOR 
    in_directory = line
    FOR i = 0, 5 DO BEGIN   ; ignore file name
        readf, settings_lun, line
    ENDFOR 
ENDIF ELSE BEGIN ; file mode
    FOR i = 0, 2 DO BEGIN 
        readf, settings_lun, line
    ENDFOR 
    in_directory = line
    FOR i = 0, 2 DO BEGIN  
        readf, settings_lun, line
    ENDFOR 
    in_file = line
    FOR i = 0, 2 DO BEGIN 
        readf, settings_lun, line
    ENDFOR 
ENDELSE 
out_directory = line
readf, settings_lun, line
readf, settings_lun, line
readf, settings_lun, read_whole_file
IF (read_whole_file NE 0) AND (read_whole_file NE 1) THEN BEGIN
    print, 'ERROR: invalid entry for read whole file'
    stop
ENDIF 
FOR i = 0, 3 DO BEGIN 
    readf, settings_lun, line
ENDFOR 
start_time = line
readf, settings_lun, line
stop_time = line
close, settings_lun
free_lun, settings_lun

IF directory_mode EQ 1 THEN BEGIN 
    cdf_files = file_search(in_directory, '*.cdf', /fully_qualify_path)
    IF (n_elements(cdf_files) GT 1) OR ((n_elements(cdf_files) EQ 1) AND (cdf_files[0] NE '')) THEN have_cdf = 1 ELSE have_cdf = 0
    IF (have_cdf EQ 1) THEN files = cdf_files ELSE BEGIN $
        print, 'Error: Directory has no .cdf files'
        stop
    ENDELSE 
    temp_names = file_basename(files)
    FOR ii = 0, n_elements(temp_names)-1 DO BEGIN
       if strlen(temp_names[ii]) NE 34 THEN BEGIN
          print, 'Error: Unexpected file in directory: '+temp_names[ii]
          return
       endif 
       temp_name = temp_names[ii]
       IF ii EQ 0 THEN file_names = [temp_name] ELSE BEGIN 
          found = 0
          FOR jj = 0, n_elements(file_names)-1 DO BEGIN
             IF file_names[jj] EQ temp_name THEN found = 1 ELSE dummy = dummy
          ENDFOR
          IF found EQ 0 THEN file_names = [file_names, temp_name]
       ENDELSE 
    ENDFOR 
 ENDIF ELSE file_names = [in_file]

header = 'Day'+tab+'Time'+tab+'Density'+tab+'Bulk Speed'+tab+$
         'Vr_HERTN'+tab+'Vt_HERTN'+tab+'Vn_HERTN'+tab+'NS_HERTN'+tab+'EW_HERTN'+tab+$
         'Vr_RTN'+tab+'Vt_RTN'+tab+'Vn_RTN'+tab+'NS_RTN'+tab+'EW_RTN'+tab+$
         'Temperature_Inst'+tab+'Pressure_Inst'+tab+'SChan_Switch'+tab+'Mom_Array'

FOR z = 0, n_elements(file_names)-1 DO BEGIN 
    in_file = strmid(file_names[z],0,30) ; take off .cdf
    instrument_id = strmid(in_file, 2, 1)
    date = strmid(in_file, 15, 8)
    cdf_version = fix(strmid(in_file, 25, 2))
    in_file_path = in_directory+'/'+in_file+'.cdf'
    cdf_id = cdf_open(in_file_path)
                                ; get epoch
    cdf_control, cdf_id, get_var_info = epoch1info, var = 'Epoch1', /zvariable
    IF epoch1info.maxrec GE 0 THEN BEGIN 
        cdf_varget, cdf_id, 'Epoch1', temp_epoch, rec_count = epoch1info.maxrec+1, /zvariable
        cdf_varget, cdf_id, 'SChannel_switch',      schan,    rec_count = epoch1info.maxrec+1, /zvariable
        cdf_varget, cdf_id, 'Moment_Array',         array,    rec_count = epoch1info.maxrec+1, /zvariable
        cdf_varget, cdf_id, 'Density',              den,      rec_count = epoch1info.maxrec+1, /zvariable
        cdf_varget, cdf_id, 'Bulk_Speed',           speed,    rec_count = epoch1info.maxrec+1, /zvariable
        cdf_varget, cdf_id, 'Vr_HERTN',             vr_hertn, rec_count = epoch1info.maxrec+1, /zvariable
        cdf_varget, cdf_id, 'Vt_HERTN',             vt_hertn, rec_count = epoch1info.maxrec+1, /zvariable
        cdf_varget, cdf_id, 'Vn_HERTN',             vn_hertn, rec_count = epoch1info.maxrec+1, /zvariable
        cdf_varget, cdf_id, 'N_S_flow_angle_HERTN', ns_hertn, rec_count = epoch1info.maxrec+1, /zvariable
        cdf_varget, cdf_id, 'E_W_flow_angle_HERTN', ew_hertn, rec_count = epoch1info.maxrec+1, /zvariable
        cdf_varget, cdf_id, 'Vr_RTN',               vr_rtn,   rec_count = epoch1info.maxrec+1, /zvariable
        cdf_varget, cdf_id, 'Vt_RTN',               vt_rtn,   rec_count = epoch1info.maxrec+1, /zvariable
        cdf_varget, cdf_id, 'Vn_RTN',               vn_rtn,   rec_count = epoch1info.maxrec+1, /zvariable
        cdf_varget, cdf_id, 'N_S_flow_angle_RTN',   ns_rtn,   rec_count = epoch1info.maxrec+1, /zvariable
        cdf_varget, cdf_id, 'E_W_flow_angle_RTN',   ew_rtn,   rec_count = epoch1info.maxrec+1, /zvariable
        cdf_varget, cdf_id, 'Temperature_Inst',     temp,     rec_count = epoch1info.maxrec+1, /zvariable
        cdf_varget, cdf_id, 'Pressure_Inst',        pressure, rec_count = epoch1info.maxrec+1, /zvariable

        if n_elements(out_directory) eq 1 then out_filename = out_directory+'/moments_'+in_file+'.txt' ELSE $
           out_filename = 'moments_'+in_file+'.txt'
        openw, out_lun, out_filename, /get_lun
        temp_string = 'Moment Data from file: '+in_file+' Instrument: '+instrument_id+$
                 ' BeaconBrowseReader Version: '+string(version)
        printf, out_lun, temp_string
        printf, out_lun, header

        converted_epoch = time_double(temp_epoch, /epoch)
        FOR ii = 0, epoch1info.maxrec DO BEGIN
            temp_time = systime(0, converted_epoch[ii], /UTC)
            temp_time_struct = bin_date(temp_time)
            temp_time_string = string(temp_time_struct[3], format = '(I2.2)')+':'+$
                               string(temp_time_struct[4], format = '(I2.2)')+':'+$
                               string(temp_time_struct[5], format = '(I2.2)')
            IF (read_whole_file EQ 1) OR ((temp_time_string GE start_time) AND (temp_time_string LE stop_time)) THEN BEGIN
               temp_string = string(temp_time_struct[1], format = '(I2.2)')+'/'+string(temp_time_struct[2], format = '(I2.2)')+'/'+$
                             string(temp_time_struct[0], format = '(I4.4)')+tab+$
                             string(temp_time_struct[3], format = '(I2.2)')+':'+$
                             string(temp_time_struct[4], format = '(I2.2)')+':'+string(temp_time_struct[5], format = '(I2.2)')+tab+$
                             string(den     [ii])+tab+string(speed   [ii])+tab+$
                             string(vr_hertn[ii])+tab+string(vt_hertn[ii])+tab+string(vn_hertn[ii])+tab+$
                             string(ns_hertn[ii])+tab+string(ew_hertn[ii])+tab+$
                             string(vr_rtn  [ii])+tab+string(vt_rtn  [ii])+tab+string(vn_rtn[ii])+tab+$
                             string(ns_rtn  [ii])+tab+string(ew_rtn  [ii])+tab+$
                             string(schan   [ii])+tab+string(array   [ii])
               printf, out_lun, temp_string
            ENDIF 
         ENDFOR 
    ENDIF
    cdf_close, cdf_id
    close,     out_lun
    free_lun,  out_lun
    print, 'finished '+in_file
ENDFOR 
print, 'done'
END 
